local uc = select(2, UnitClass("player"))
if (uc ~= "HUNTER") and (uc ~= "ROGUE") then return end

MDH = LibStub("AceAddon-3.0"):NewAddon("MDH", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MisDirectionHelper",true)
local QTC = LibStub("LibQTip-1.0")
local icon = LibStub("LibDBIcon-1.0")
local _G = _G
local MDHText = "MDH"
local channels = {["RAID"] = _G.RAID, ["PARTY"] = _G.PARTY, ["WHISPER"] = _G.WHISPER}
local misdtarget, tooltip
local imd = GetSpellInfo(34477)
local iff = GetSpellInfo(82692)
local itt = GetSpellInfo(57934)
local iua = GetSpellInfo(51672)
local hiconinfo = {
	[imd] = {"Interface\\Icons\\Ability_Hunter_Misdirection", 151},
	[iff] = {"Interface\\Icons\\Ability_Hunter_FocusFire", 134},
}
local hicons = {[151] = imd, [134] = iff}
local riconinfo = {
	[itt] = {"Interface\\Icons\\Ability_Rogue_TricksOftheTrade", 454},
	[iua] = {"Interface\\Icons\\Ability_Rogue_UnfairAdvantage", 457},
}
local ricons = {[454] = itt, [457] = iua}
local UnitName = UnitName
local GetPartyAssignment = GetPartyAssignment
local UnitGetAvailableRoles = UnitGetAvailableRoles
local UnitGroupRolesAssigned = UnitGroupRolesAssigned
local UnitHealthMax = UnitHealthMax

local function set(info, value)

	local key = info[#info]
	MDH.db.profile[key] = value

end

local function get(info)

	local key = info[#info]
	return MDH.db.profile[key]

end

local defaults = {
	profile = {
		minimap = {hide = false},
		cChannel = "PARTY",
		name = "Pet",
		petname = _G.UNKNOWN,
		bAnnounce = false,
		hicon = 151,
		ricon = 454,
		hname = "MDH " .. imd,
		rname = "MDH " .. itt,
		target = "pet",
		target2 = nil,
		name2 = nil,
		clearjoin = nil,
		remind = nil,
	},	
}

--Create the data broker
function MDH:CreateLDBObject()

	MDH.dataObject = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("MisdirectionHelper", {
		type = "data source",
		text = MDH:TTText("both"),
		icon = "Interface\\Icons\\Ability_Hunter_Misdirection",
		OnClick = 
			function(frame, button)
				if button == "RightButton" then
					InterfaceOptionsFrame_OpenToCategory(MDH.optionsFrame)
				elseif button == "LeftButton" then
					MDH:MDHtarget(button)
				end
			end,
		OnEnter = 
			function(frame)
				tooltip = QTC:Acquire("Broker_MisdirectionHelperTooltip", 1, "CENTER")
				tooltip:SmartAnchorTo(frame)
				tooltip:SetAutoHideDelay(0.25, frame)
				MDH:MDHShowToolTip()
			end,
		OnLeave = nil,
	})
	
end

local function HandlerFunc(arg, button)

    if arg == "b1" then

		MDH:PrtyTank(button)

    elseif arg == "b2" then
	
		MDH:MDHtarget(button)

    elseif arg == "b3" then

		MDH:ToT(button)

	elseif arg == "b4" then

		MDH:PlyrPet(button)

	elseif arg == "b5" then

		MDH:PopUp(button)

	elseif arg == "f" then
	
		MDH:MDHfocus(button)
		
	elseif arg == "b6" then

		InterfaceOptionsFrame_OpenToCategory(MDH.optionsFrame)

	elseif arg == "c" then
	
		MDH:ClearTarget(button)

    end

end

function MDH:ClearTarget(button)

	if button == "LeftButton" then
	
		self.db.profile.target = nil
		self.db.profile.name = nil
		
	else
	
		self.db.profile.target2 = nil
		self.db.profile.name2 = nil
		
	end
	
	self:MDHEditMacro()
	self:MDHShowToolTip()

end

function MDH:MDHShowToolTip()

	if InCombatLockdown() then return end

	local headerFont, y
	
	if not tooltip then return end
	if not tooltip.lines then return end

	tooltip:Clear()
	
	headerFont = CreateFont("MisdirectionHelperHeaderFont")
	MisdirectionHelperHeaderFont:CopyFontObject(GameTooltipHeaderText)
	MisdirectionHelperHeaderFont:SetFont(MisdirectionHelperHeaderFont:GetFont(), 15)
	
    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "|cffffff00MisdirectionHelper|r",MisdirectionHelperHeaderFont, "CENTER", 1)
    tooltip:SetLineColor(y, 1, 0, 1, 1)
	
    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
    tooltip:SetLineColor(y, 0, 0, 1, 1)

    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, " " .. L["Set To Party Tank"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("b1", button) end)
    tooltip:SetLineColor(y, 1, 0, 0, 1)
	
    y = tooltip:AddLine()
    tooltip:SetCell(y, 1, " " .. L["Set To Target"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("b2", button) end)
    tooltip:SetLineColor(y, 1, 0, 0, 1)
	
    y = tooltip:AddLine()
    tooltip:SetCell(y, 1, " " .. L["Set to Target Of Target"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("b3", button) end)
    tooltip:SetLineColor(y, 1, 0, 0, 1)

	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, " " .. L["Set to Focus"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("f", button) end)
	tooltip:SetLineColor(y, 1, 0, 0, 1)
	
    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	if uc == "HUNTER" then
	
		y = tooltip:AddLine()
		tooltip:SetCell(y, 1, " " .. L["Set to Your Pet"])
		tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("b4", button) end)
		tooltip:SetLineColor(y, 0, 1, 1, 1)
		
	end

    y = tooltip:AddLine()
    tooltip:SetCell(y, 1, " " .. L["Enter Player Name"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("b5", button) end)
	tooltip:SetLineColor(y, 0, 1, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, " " .. L["Clear Target"])
	tooltip:SetCellScript(y, 1, "OnMouseDown", function(this, arg, button) HandlerFunc("c", button) end)
	tooltip:SetLineColor(y, 0, 0.8, 0.2, 1)

	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
    tooltip:SetCell(y, 1, _G.OPTIONS, "b6")
    tooltip:SetLineColor(y, 0, 1, 0, 1)
	
    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	 
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "|cffffff00" .. string.format(L["Left click target: %s"], "|cffff0033 " .. self:TTText("left")), "CENTER", 1)
	tooltip:SetLineColor(y, 1, 0, 1, 0)
	
	if self.db.profile.target2 then
	
		y = tooltip:AddLine()
		tooltip:SetCell(y, 1, "|cffffff00" .. string.format(L["Right click target: %s"], "|cffff0033" .. self:TTText("right")), "CENTER", 1)
		tooltip:SetLineColor(y, 1, 0, 1, 0)
		
	end
	
    y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", "CENTER", 1)
	tooltip:SetLineColor(y, 0, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "|cffffff00" .. L["Left Click Each Text To Activate"] .. "|r ", "CENTER", 1)
	tooltip:SetLineColor(y, 1, 0, 1, 1)
	
	y = tooltip:AddLine()
	tooltip:SetCell(y, 1, "|cffffff00" .. L["Right Click For MDH Options"] .. "|r", "CENTER", 1)
    tooltip:SetLineColor(y, 1, 0, 1, 1)
	
    tooltip:Show()

end

function MDH:MDHTextUpdate()

	MDH.dataObject.text = self:TTText("both")

end

function MDH:validateTarget(unit)

	local t = UnitName(unit)
	local mdt = unit

	if t then
	
		if UnitIsUnit(unit, "pet") or UnitInParty(unit) or UnitInRaid(unit) then
		
			mdt = t
			
		end
		
	end
	
	return mdt
	
end

function MDH:ToT(button)
	
	if button == "LeftButton" then
	
		self.db.profile.target = "targettarget"
		self.db.profile.name = self:validateTarget("targettarget")
		
	else

		self.db.profile.target2 = "targettarget"
		self.db.profile.name2 = self:validateTarget("targettarget")
	
	end
	
	self:MDHEditMacro()
	self:MDHShowToolTip()
	
end

function MDH:PlyrPet(button)

	if UnitExists("pet") then
	
		self:MDHgetpet()

	end
	
	if button == "LeftButton" then
	
		self.db.profile.target = "pet"
		self.db.profile.name = self.db.profile.petname
		
	else
	
		self.db.profile.target2 = "pet"
		self.db.profile.name2 = self.db.profile.petname

	end

	self:MDHEditMacro()
	self:MDHShowToolTip()
	
end

function MDH:PrtyTank(button)

	self:MDHtank(button)
	self:MDHEditMacro()
	self:MDHShowToolTip()
	
end

function MDH:PopUp(button)

	self.button = button
	StaticPopup_Show("MDH_GET_PLAYER_NAME")

end

function MDH:MDHLoad()

	if UnitExists("pet") then

		self:MDHgetpet()
		
	end

	self:MDHEditMacro()

end

function MDH:MDHgetpet()

	local petname = UnitName("pet")

	if petname == _G.UNKNOWN then
	
		petname = self.db.profile.petname
		
	end

	self.db.profile.petname = petname

end

function MDH:MDHfocus(button)

	if button == "LeftButton" then
	
		self.db.profile.target = "focus"
		self.db.profile.name = self:validateTarget("focus")
		
	else
	
		self.db.profile.target2 = "focus"
		self.db.profile.name2 = self:validateTarget("focus")
		
	end
	
	self:MDHEditMacro()
	self:MDHShowToolTip()
	
end

function MDH:MDHtarget(button)
	
	if button == "LeftButton" then
	
		self.db.profile.target = self:validateTarget("target")
		self.db.profile.name = self:validateTarget("target")
		
	else
	
		self.db.profile.target2 = self:validateTarget("target")
		self.db.profile.name2 = self:validateTarget("target")
		
	end
	
	self:MDHEditMacro()
	self:MDHShowToolTip()

end

function MDH:MDHtank(button)

	local members = GetNumRaidMembers()
	local prefix = "raid"
	local mt, mth, name, tank, canBeTank, member, maxHealth
	
	if members == 0 then
	
		members = GetNumPartyMembers()
		prefix = "party"
		
	end
	
	if members == 0 then 
	
		--print("not in a group")
		return 
		
	end
	
	for member = 1, 40 do
	
		name = UnitName(prefix .. member)
		
		if name then
		
			tank = GetPartyAssignment("MAINTANK", name) or (UnitGroupRolesAssigned(name) == "TANK")
			
			if not tank then
			
				canBeTank = UnitGetAvailableRoles(name)
				
				if canBeTank then
				
					maxHealth = UnitHealthMax(name)
					
					if maxHealth > (mth or 0) then
					
						mth = maxHealth
						mt = name
						
					end
					
				end
				
			else
			
				mt = name
				break
			
			end
			
		end
		
	end

	if mt then

		--print("Tank found:", mt)
		if button == "LeftButton" then
		
			self.db.profile.target = mt
			self.db.profile.name = mt
			
		else
		
			self.db.profile.target2 = mt
			self.db.profile.name2 = mt
			
		end
	
	else
	
		--print("No tank found")

	end

end

function MDH:MDHEditMacro()

	local singlemacro, multiplemacro, macro, macroid
	local spell = imd
	local id = MDH.db.profile.hicon or hiconinfo[imd][2]
	local mname = MDH.db.profile.hname

	if uc == "ROGUE" then
	
		spell = itt
		id = MDH.db.profile.ricon or riconinfo[itt][2]
		mname = MDH.db.profile.rname

	end
	
	self:MDHTextUpdate()
	
	singlemacro = "#showtooltip\n/cast [mod:shift,@none][@%s,nodead,exists] %s; %s"
	multiplemacro = "#showtooltip\n/cast [mod:shift,@none][btn:1,@%s,nodead,exists][btn:2,@%s,nodead,exists] %s; %s"

	if self.db.profile.target2 then
	
		macro = string.format(multiplemacro, self.db.profile.target or "target", self.db.profile.target2 or "target", spell, spell)
		
	else
	
		macro = string.format(singlemacro, self.db.profile.target or "target", spell, spell)
		
	end
	
	macroid = GetMacroIndexByName(mname)

	if macroid == 0 then
	
		CreateMacro(mname , id, macro, 1, 1)

	else

		EditMacro(macroid, mname , id, macro)

	end

end

function MDH:MDHChat()

	local msg = string.format((uc == "HUNTER") and L["%s Misdirects to %s"] or L["%s casts Tricks of the Trade on %s"], UnitName("player"), misdtarget)
	local chan = self.db.profile.cChannel or "RAID"
	local s = false
	
	if IsAddOnLoaded("CastYeller2") or IsAddOnLoaded("CastYeller") then return end

	if (chan == "PARTY") and (GetNumPartyMembers() ~= 0) then
	
		s = true

	elseif (chan == "RAID") and (GetNumRaidMembers() ~= 0) then

		s = true
		
	elseif chan == "WHISPER" then
	
		s = true

	end
	
	if s then SendChatMessage(msg, chan, nil, misdtarget) end

end

--First visible options frame
local function createMainPanel()

	local frame = CreateFrame("Frame", "MisdirectionHelperMain")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local version = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local author = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")

	title:SetFormattedText("|T%s:%d|t %s", "Interface\\ICONS\\Ability_Hunter_Misdirection", 32, "Misdirection Helper 2")
	title:SetPoint("CENTER", frame, "CENTER", 0, 170)

	version:SetText(_G.GAME_VERSION_LABEL .. " " .. GetAddOnMetadata("MisdirectionHelper2", "Version"))
	version:SetPoint("CENTER", frame, "CENTER", 0, 130)

	author:SetText(L["Author"] .. ": Deepac")
	author:SetPoint("CENTER", frame, "CENTER", 0, 100)

	return frame

end

function MDH:OnInitialize()
	
	local AceConfig = LibStub("AceConfig-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
	local AceDBOptions = LibStub("AceDBOptions-3.0")
	local mainPanel = createMainPanel()
	local optionsTable

	self.db = LibStub("AceDB-3.0"):New("MisDirectionHelperDB", defaults)
	optionsTable = {
		type = "group",
		name = _G.MAIN_MENU,
		inline = false,
		args = {
			showMinimapIcon = {
				order = 1,
				type = "toggle",
				width = "full",
				name = L["Minimap Icon"],
				desc = L["Toggle Minimap icon"],
				get = function() return not MDH.db.profile.minimap.hide end,
				set = function(info, value)
					MDH.db.profile.minimap.hide = not value
					if value then
						icon:Show("MisdirectionHelper")
					else
						icon:Hide("MisdirectionHelper")
					end
				end,
			},
			clearleave = {
				order = 2,
				type = "toggle",
				width = "full",
				name = L["Clear when joining group"],
				desc = L["Clear both targets when joining a raid / party"],
				get = function() return MDH.db.profile.clearjoin end,
				set = function(info, value)
					MDH.db.profile.clearjoin = value
					if value then
						MDH:RegisterEvent("PARTY_MEMBERS_CHANGED")
					else
						MDH:UnregisterEvent("PARTY_MEMBERS_CHANGED")
					end
				end,
			},
			remind = {
				order = 3,
				type = "toggle",
				width = "full",
				name = L["Show reminder"],
				desc = L["Display a reminder to set targets on entering an instance / raid"],
				get = function() return MDH.db.profile.remind end,
				set = function(info, value)
					MDH.db.profile.remind = value
					if value then
						MDH:RegisterEvent("ZONE_CHANGED_NEW_AREA")
					else
						MDH:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
					end
				end,
			},
			buffmessage = {
				order = 4,
				type = "toggle",
				name = L["Buff Alert"],
				desc = L["Toggle Misdirection Applied Announcment"],
				get = function() return MDH.db.profile.bAnnounce end,
				set = function(info, value)
					MDH.db.profile.bAnnounce = value
				end,
			},
			cChannel = {
				type = "select",
				name = L["Buff Channel"],
				desc = L["Select Channel for Buff Announcements"],
				order = 5,
				get = function() return MDH.db.profile.cChannel end,
				set = function(info, value)
					MDH.db.profile.cChannel = value
				end,
				values = channels,
			},
			spacer1 = {
				order = 6,
				type = "description",
				name = "\n",
			},			
			hicon = {
				type = "select",
				name = L["Misdirection macro icon"],
				order = 7,
				hidden = function() return uc == "ROGUE" end,
				get = function() return MDH.db.profile.hicon or hiconinfo[imd][2] end,
				set = function(info, value)
					MDH.db.profile.hicon = value
					MDH:MDHEditMacro()
				end,
				values = hicons,
			},
			ricon = {
				type = "select",
				name = L["Tricks of the Trade macro icon"],
				order = 8,
				hidden = function() return uc == "HUNTER" end,
				get = function() return MDH.db.profile.ricon or riconinfo[itt][2] end,
				set = function(info, value)
					MDH.db.profile.ricon = value
					MDH:MDHEditMacro()
				end,
				values = ricons,
			},
			spacer2 = {
				order = 9,
				type = "description",
				name = "\n",
			},
			hmname = {
				type = "input",
				name = L["Macro name"],
				width = "double",
				order = 10,
				desc = L["Name to use for the macro"],
				get = function() return MDH.db.profile.hname end,
				set = function(info, value)
					MDH.db.profile.hname = value
					MDH:MDHEditMacro()
				end,
				hidden = function() return uc == "ROGUE" end,
			},
			rmname = {
				type = "input",
				name = L["Macro name"],
				desc = L["Name to use for the macro"],
				order = 11,
				width = "double",
				get = function() return MDH.db.profile.rname end,
				set = function(info, value)
					MDH.db.profile.rname = value
					MDH:MDHEditMacro()
				end,
				hidden = function() return uc == "HUNTER" end,
			},
		},
	}

	--remove old renamed variables
	self.db.profile.Name = nil
	self.db.profile.Petname = nil

	mainPanel.name = "Misdirection Helper 2"
	InterfaceOptions_AddCategory(mainPanel)
	AceConfig:RegisterOptionsTable("MisdirectionHelperOptions", optionsTable)
	AceConfig:RegisterOptionsTable("MisdirectionHelperProfiles", AceDBOptions:GetOptionsTable(self.db))
	MDH.optionsFrame = AceConfigDialog:AddToBlizOptions("MisdirectionHelperOptions", _G.MAIN_MENU, "Misdirection Helper 2")
	AceConfigDialog:AddToBlizOptions("MisdirectionHelperProfiles", L["Profiles"], "Misdirection Helper 2")
	
	self:CreateLDBObject()

	if icon then

		icon:Register("MisdirectionHelper", MDH.dataObject, MDH.db.profile.minimap)

	end
	
	if (GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0) or (UnitInRaid("player")) then
	
		self.ingroup = true
		
	end

	self:MDHOnload()

end

function MDH:MDHOnload()

	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("UNIT_PET")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	--self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED")
	
	if self.db.profile.clearleave then
	
		self:RegisterEvent("PARTY_MEMBERS_CHANGED")
		
	end

	if self.db.profile.remind then
	
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		
	end

end

--this should only be called when the relevant option is set
function MDH:PARTY_MEMBERS_CHANGED()

	local inParty, inRaid

	inParty = GetNumPartyMembers()
	inRaid = GetNumRaidMembers()

	if (inParty > 0) or (inRaid > 0) or (UnitInRaid("player")) then
	
		if not self.ingroup then
		
			self:ClearTarget("LeftButton")
			self:ClearTarget("RightButton")
			self.ingroup = true
			print("|cff568bffMisdirection Helper:|r " .. L["Targets reset"])
	
		end
	
	else
	
		self.ingroup = nil

	end

end

function MDH:ZONE_CHANGED_NEW_AREA()

	local inInstance = (select(2, GetInstanceInfo()) ~= "none")
	
	--don't do anything if the char is dead
	if UnitIsGhost("player") then return end
	
	if inInstance then
	
		if self.remind then
	
			--reminder already set, don't give another one
		
		else
		
			StaticPopup_Show("MDH_REMINDER")
			self.remind = true
			
		end
		
	else
	
		self.remind = nil
		
	end
	
end

function MDH:PLAYER_TARGET_CHANGED()

	if not InCombatLockdown() then
	
		if self.db.profile.target == "target" then
		
			self.db.profile.name = self:validateTarget("target")
			self:MDHTextUpdate()
			
		end
	
	end
	
end

function MDH:PLAYER_FOCUS_CHANGED()

	if not InCombatLockdown() then
	
		if self.db.profile.target == "focus" then
			
			self.db.profile.name = self:validateTarget("focus")
			self:MDHTextUpdate()
			
		end
	
	end
	
end

function MDH:PLAYER_ENTERING_WORLD()

	self:MDHLoad()

end

function MDH:UNIT_PET(event, unitid)

	local pet
	
	if unitid == "player" and UnitExists("pet") then
 
		pet = UnitName("pet")

		if pet ~= self.db.profile.petname then

			self:MDHgetpet()
			
		end
		
		if self.db.profile.target == "pet" then
		
			self.db.profile.name = self.db.profile.petname
			
		elseif self.db.profile.target2 == "pet" then
		
			self.db.profile.name2 = self.db.profile.petname
			
		end
		
		self:MDHTextUpdate()
		
	end

end

function MDH:UNIT_SPELLCAST_SENT(event, unitid, spell, rank, target, ...)

	if unitid == "player" then

		if uc == "HUNTER" then
		
			if spell == imd then
	
				misdtarget = target
				
			end
			
		elseif uc == "ROGUE" then
		
			if spell == itt then
			
				misdtarget = target
				
			end
			
		end

	end

end

function MDH:UNIT_SPELLCAST_SUCCEEDED(event, unitid, spell, rank, ...)

	local cast

	if unitid == "player" then

		if uc == "HUNTER" then
		
			if spell == imd then
			
				cast = true

			end
			
		elseif uc == "ROGUE" then
		
			if spell == itt then
			
				cast = true
				
			end
			
		end
		
		if cast and self.db.profile.bAnnounce then

			self:MDHChat()

		end
		
	end
	
end

function MDH:PLAYER_REGEN_DISABLED()

	--unit entered combat
	if tooltip then tooltip:Hide() end
	
end

function MDH:TTText(t)

	local n = self.db.profile.name or ""
	local ts, te, target
	local uname, out
	
	if t == "left" then ts = 1; te = 1 end
	if t == "right" then ts = 2; te = 2 end
	if t == "both" then ts = 1; te = 2 end

	for target = ts, te do

		if target == 2 then 
		
			n = self.db.profile.name2
			
			if not n then break end
			
		end

		if out then
		
			out = out .. ", "
			
		end
		
		if n == "target" then
		
			uname = _G.TARGET
			
		elseif n == "targettarget" then
		
			uname = _G.SHOW_TARGET_OF_TARGET_TEXT
			
		elseif n == "focus" then
		
			uname = _G.BINDING_NAME_FOCUSTARGET
			
		elseif n == "pet" then
		
			uname = _G.PET
			
		elseif n == "" then
		
			uname = L["Nobody"]
	
		else
	
			uname = n

		end
	
		out = (out or "") .. uname
	
	end
	
	return out

end

local function onAccept(this)

	local button = self.button
	local t
	
	if not UnitAffectingCombat("player") then

		if button == "LeftButton" then
		
			t = string.lower(this.editBox:GetText() or "")
			MDH.db.profile.target = t
			MDH.db.profile.name = t
			
		else
		
			t = string.lower(this.editBox:GetText() or "")
			MDH.db.profile.target2 = t
			MDH.db.profile.name2 = t

		end
		
		if t == "pet" then
		
			if button == "LeftButton" then
			
				MDH.db.profile.name = MDH.db.profile.petname
				
			else
			
				MDH.db.profile.name2 = MDH.db.profile.petname
				
			end
			
		elseif t == "tank" then
		
			MDH:MHDtank(button)
			
		elseif (t == "target") or (t == "t") then
		
			MDH:MHDtarget(button)

		end
		
		MDH:MDHEditMacro()
		
	end

	MDH:MDHShowToolTip()
	
	this:Hide()
	
end

StaticPopupDialogs["MDH_GET_PLAYER_NAME"] = {
	text = (uc == "HUNTER") and L["Player to Misdirect to"] or L["Player to cast Tricks of the Trade on"],
	button1 = _G.ACCEPT,
	button2 = _G.CANCEL,
	hasEditBox = 1,
	OnAccept = onAccept,
	EditBoxOnEnterPressed = function(this) onAccept(this:GetParent()) end,
	EditBoxOnEscapePressed = function(this) this:GetParent():Hide()	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["MDH_REMINDER"] = {
	text = L["Set Misdirection Helper's targets!"],
	button1 = _G.OKAY,
	OnAccept = function(this) this:Hide() end,
	timeout = 0,
	whileDead = 0,
	hideOnEscape = 1,
}